@@ -1,4 +1,5 @@
 use crate::*;
+use page::PAGE_SIZE;
 use std::collections::hash_map::Entry;
 
 /// Sorted Record storage.
@@ -119,7 +120,7 @@ impl SortedFile {
         if x != 0 {
             self.free_parent_node(db, p, p.left(x), r);
             self.free_parent_node(db, p, p.right(x), r);
-            r.drop_key(db, &p.data[p.rec_offset(x)..]);
+            p.drop_key(db, x, r);
             let cp = p.child_page(x);
             if p.level > 1 {
                 self.free_page(db, cp, r);
@@ -277,6 +278,10 @@ impl SortedFile {
         self.pages.borrow_mut().insert(pnum, pp);
     }
 
+    fn remove_page(&self, pnum: u64) {
+        self.pages.borrow_mut().remove(&pnum);
+    }
+
     /// Get a page from the cache, or if it is not in the cache, load it from external storage.
     fn load_page(&self, db: &DB, pnum: u64) -> PagePtr {
         if !self.ok.get() {
@@ -323,6 +328,111 @@ impl SortedFile {
             self.dirty_pages.borrow_mut().push(pp.clone());
         }
     }
+
+    /// Attempt to free up logical pages by re-packing child pages.
+    pub fn repack(&self, db: &DB, r: &dyn Record) -> i64 {
+        self.repack_page(db, self.root_page, r)
+    }
+
+    /// Repack a page. Result is number of pages freed.
+    fn repack_page(&self, db: &DB, pnum: u64, r: &dyn Record) -> i64 {
+        let mut result = 0;
+
+        let pp = self.load_page(db, pnum);
+        let p = &mut pp.borrow_mut();
+
+        if p.level == 0 {
+            return 0;
+        }
+
+        if p.level > 1 {
+            result += self.repack_page(db, p.first_page, r);
+            if result >= REPACK_LIMIT {
+                return result;
+            }
+            result += self.repack_children(db, p, p.root, r);
+            if result >= REPACK_LIMIT {
+                return result;
+            }
+        }
+
+        let (x, y) = self.page_total(db, p, p.root, r);
+        let n = 1 + x;
+        if n < 2 {
+            return result;
+        }
+        let total = y + db.page_size(p.first_page);
+        let full = (n * PAGE_SIZE) as u64;
+        let space = full - total;
+        if space < full / 10 {
+            return result;
+        }
+
+        self.set_dirty(p, &pp);
+
+        // Iterate over the page child records. Append them into a list of new pages.
+        let mut plist = PageList::default();
+        plist.add(db, p.first_page, r, self);
+        self.move_children(db, p, p.root, r, &mut plist);
+
+        if TRACE_PACK {
+            println!(
+                "pnum={} level={} #children={} total={} full={} space={}",
+                pnum,
+                p.level,
+                n,
+                total,
+                full,
+                full - total
+            );
+            println!(
+                "new #children={} record count={}",
+                plist.list.len(),
+                plist.count
+            );
+        }
+        result += plist.store_to(db, p, self);
+        result
+    }
+
+    /// Count number child pages and their total size, to decide whether to repack a parent page.
+    fn page_total(&self, db: &DB, p: &Page, x: usize, r: &dyn Record) -> (usize, u64) {
+        if x == 0 {
+            return (0, 0);
+        }
+        let cp = p.child_page(x);
+        let cp_size = db.page_size(cp);
+        let (n1, t1) = self.page_total(db, p, p.left(x), r);
+        let (n2, t2) = self.page_total(db, p, p.right(x), r);
+        (1 + n1 + n2, cp_size + t1 + t2)
+    }
+
+    /// Move child records into PageList.
+    fn move_children(&self, db: &DB, p: &Page, x: usize, r: &dyn Record, plist: &mut PageList) {
+        if x != 0 {
+            self.move_children(db, p, p.left(x), r, plist);
+            let cp = p.child_page(x);
+            plist.add(db, cp, r, self);
+            p.drop_key(db, x, r);
+            self.move_children(db, p, p.right(x), r, plist);
+        }
+    }
+
+    fn repack_children(&self, db: &DB, p: &Page, x: usize, r: &dyn Record) -> i64 {
+        let mut result = 0;
+        if x != 0 {
+            result += self.repack_page(db, p.child_page(x), r);
+            if result >= REPACK_LIMIT {
+                return result;
+            }
+            result += self.repack_children(db, p, p.left(x), r);
+            if result >= REPACK_LIMIT {
+                return result;
+            }
+            result += self.repack_children(db, p, p.right(x), r);
+        }
+        result
+    }
 } // end impl SortedFile
 
 /// Used to pass parent page number for insert operations.
@@ -422,6 +532,7 @@ impl Asc {
         result
     }
 }
+
 impl Iterator for Asc {
     type Item = (PagePtr, usize);
 
@@ -606,3 +717,76 @@ impl Stack {
         }
     }
 } // end impl Stack
+
+/// PageList is used to implement repacking of child pages ( REPACKFILE builtin funcction ).
+#[derive(Default)]
+pub struct PageList {
+    pub cur: usize,
+    pub list: Vec<(Page, Option<Box<dyn Record>>)>,
+    pub pnums: Vec<u64>,
+    pub count: usize,
+}
+
+const TRACE_PACK: bool = true;
+
+/// Limit on how many pages to free in one transaction.
+const REPACK_LIMIT: i64 = 10;
+
+impl PageList {
+    fn store_to(&mut self, db: &DB, p: &mut Page, file: &SortedFile) -> i64 {
+        let mut pnums = std::mem::take(&mut self.pnums);
+        let list = std::mem::take(&mut self.list);
+        p.clear();
+
+        for (cp, key) in list {
+            let cpnum = pnums.pop().unwrap();
+            file.publish_page(cpnum, cp);
+            if let Some(key) = key {
+                p.append_page(&*key, cpnum);
+            } else {
+                p.first_page = cpnum;
+                if TRACE_PACK {
+                    println!("first page set to {}", cpnum);
+                }
+            }
+        }
+
+        if TRACE_PACK {
+            println!("Free pages from pack={:?}", pnums);
+        }
+        let result = pnums.len();
+        while let Some(pnum) = pnums.pop() {
+            db.free_page(pnum);
+        }
+        result as i64
+    }
+
+    fn add(&mut self, db: &DB, pnum: u64, r: &dyn Record, file: &SortedFile) {
+        let pp = file.load_page(db, pnum);
+        file.remove_page(pnum);
+        let p = &mut pp.borrow_mut();
+        if self.list.is_empty() {
+            self.list.push((p.new_page(), None));
+        }
+        self.append(db, p, p.root, r);
+        self.pnums.push(pnum);
+        p.pnum = u64::MAX;
+    }
+
+    fn append(&mut self, db: &DB, p: &Page, x: usize, r: &dyn Record) {
+        if x != 0 {
+            self.append(db, p, p.left(x), r);
+            let mut ap = &mut self.list[self.cur].0;
+            if ap.full() {
+                // start a new page
+                let key = p.get_key(db, x, r);
+                self.list.push((p.new_page(), Some(key)));
+                self.cur += 1;
+                ap = &mut self.list[self.cur].0;
+            }
+            ap.append_from(p, x);
+            self.count += 1;
+            self.append(db, p, p.right(x), r);
+        }
+    }
+}
